from setuptools import setup, find_packages

setup(
    name='reflowrst',
    packages=find_packages(),
    version='1.0.7',
    description='Modify valid rst text to fit within given width',
    long_description=open('README.rst').read(),
    author='doakey3',
    author_email='reflowrst.dmodo@spamgourmet.com',
    url='https://github.com/doakey3/reflowrst',
    download_url='https://github.com/doakey3/reflowrst/tarball/1.0.7',
    license='MIT',
)
